// Copyright (c) 2003-2008 Synology Inc. All rights reserved.
#ifndef __SYNO_SATA_H_
#define __SYNO_SATA_H_

/*
* We use g_internal_hd_num this variable pass from uboot for determine whether wake up in sequence.
* because if we need power in sequence at booting, 
* it's mean we also need wake up in sequence for power issue
*
* For old product, they don't passing g_internal_hd_num from u-boot, but in new kernel it had defined.
* so the default value is -1, it will still doing original job. So this define can compatible to old platform.
*
* I put the g_internal_hd_num check in the sata driver instaed of this. Because we only need to check in
* queuecommand. Others is just callbacks. We don't need it really.
*
* -1 : no specify. Always do spinup delay
*  0 : do not spinup delay
* >0 : The number that we would delay
*/
#ifdef MY_ABC_HERE
extern long g_internal_hd_num;

/**
 * Only for 7042. this is for 2.6.15 only.
 * @param channelIndex channel index. not bus id!!
 * 
 * @return int
 */
static inline int IsPowerOnSequence(unsigned char channelIndex)
{
	int res = 0;
	if ((g_internal_hd_num < 0) || // not specified in boot command line
		(channelIndex < g_internal_hd_num)) {
		res = 1;
	}
	return res;
}

#endif /* MY_ABC_HERE */

#ifdef MY_ABC_HERE

/* 
 *back porting from linux 2.6.28. add SYNO prefix in order to not mixed with libata
 */
#define SYNO_ATA_ID_MAJOR_VER	 80
#define SYNO_ATA_ID_MINOR_VER	 81
#define SYNO_ATA_ID_COMMAND_SET_1 82
#define SYNO_ATA_ID_COMMAND_SET_2 83
#define SYNO_ATA_ID_CFSSE		 84
#define SYNO_ATA_ID_ROT_SPEED	 217

/**
 * Determind the ata version.
 * 
 * Copy from ata.h
 * 
 * @param id     [IN] Should not be NULL. ata identify buffer.
 * 
 * @return ata version
 */
static inline unsigned int
ata_major_version(const unsigned short *id)
{
	unsigned int mver;

	if (id[SYNO_ATA_ID_MAJOR_VER] == 0xFFFF)
		return 0;

	for (mver = 14; mver >= 1; mver--)
		if (id[SYNO_ATA_ID_MAJOR_VER] & (1 << mver))
			break;
	return mver;
}

/**
 * Determind the ata version.
 * 
 * Copy from linux-2.6.28 later in ata.h. Original from mail 
 * list. But it has bug. So i customized it. 
 *  
 * Sometime you can't just only take care in major version. 
 * The actually ATA version might need to look minor version. 
 * Please refer smartmontools-5.38/atacmds.cpp 
 * const char minor_str []  = ...
 * 
 * @param id     [IN] Should not be NULL. ata identify buffer.
 * 
 * @return ata version
 */
static inline int 
ata_id_is_ssd(const unsigned short *id)
{
	int res = 0;
	unsigned int major_id = ata_major_version(id);

	/* ATA8-ACS version 4c or higher (=> 4c or 6 at the moment) */
	if (7 <= major_id){
		if (id[SYNO_ATA_ID_ROT_SPEED] == 0x01) {
			// intel ssd, and the laters ssd
			res = 1;
			goto END;
		}
	}

	if ((id[SYNO_ATA_ID_COMMAND_SET_2]>>14) == 0x01 &&
		!(id[SYNO_ATA_ID_COMMAND_SET_1] & 0x0001)) {
		// not support smart. like innodisk
		res = 1;
		goto END;
	}

	// transcend. Not support smart error log
	if ((id[SYNO_ATA_ID_COMMAND_SET_2]>>14) == 0x01 &&
		(id[SYNO_ATA_ID_COMMAND_SET_1] & 0x0001) &&
		!(id[SYNO_ATA_ID_CFSSE] & 0x1)) {
		res = 1;
		goto END;
	}

END:
	return res;
}
#endif

#endif /* __SYNO_SATA_H_ */
